[manpage_begin aitcs n 1.5]
[require   Tcl 8.6]
[require   aitcs [opt 1.5]]
[moddesc   {AitCS}]
[titledesc {Array in the Command's Shadow}]
[category  {Programming tools}]
[copyright {2020 Stuart Cassoff <stwo@users.sourceforge.net>}]
[keywords  array command namespace alias snake eagle]

[description]

	An Array in the Command's Shadow is an array with an
	associated set of commands that each provide access to
	or allow certain useful manipulations of the array.

[para]

	An AitCS may be used where a program would
	normally use global or namespace variables to hold
	configuration, state, widget paths, temp values, etc.

[para]

	Arrays are named+randomly-named and created in a
	namespace. Several commands (via aliases) are created in a
	(possibly different) namespace to manipulate the array.

[para]

	The name ends up being used much like the set command.

[para]

	The inspiration came from being tired of
	having to use the variable command in almost all
	the procs in a namespace when often in a
	given proc there is interest only in a few
	items in the global state.

[para]

	By reducing array manipulation clutter this
	extension may help in improving program clarity!


[section {COMMANDS}]

[usage namespace import ::aitcs::aitcs]

[list_begin definitions]

	[call [cmd {aitcs name}] [arg name] [opt [arg variable_namespace]]]
		Names a new array in the namespace [arg variable_namespace].
		The format of the name is "namespace::__name__cccccc_"
		where "cccccc" is six random characters in the range a-z.
		If [arg variable_namespace] isn't supplied or
		is an empty string then the array will be named in
		the caller's namespace. Doesn't create an array variable.
		Returns the name of the newly named array.

	[call [cmd {aitcs new}] [opt [arg name]] [opt [arg command_namespace]] [opt [arg variable_namespace]] [opt [arg {key val...}]]]
		Creates a new array in the namespace [arg variable_namespace]
		and creates commands (via aliases) in the [arg command_namespace] to
		manipulate the array. The format of the name and the
		[arg variable_namespace] will be as described in the
		[cmd name] ensemble subcommand. If the [arg name] isn't supplied
		or is an empty string then the [arg name] will be set to a string of six
		random characters in the range a-z. If the [arg command_namespace]
		isn't supplied or is an empty string then the
		[arg command_namespace] will be set to the [arg variable_namespace].
		The commands (via aliases) are created in the [arg command_namespace].
		Initial values are set from [arg {key val...}] if supplied.
		See the AITCS COMMANDS section.
		Creates an array variable in the [arg variable_namespace].
		Returns the name of the newly named array, or the randomly-generated
		[arg name] if the [arg name] argument wasn't supplied.

[list_end]


[section {AITCS COMMANDS}]

	Commands (via aliases) are created in the [arg command_namespace] and
	are based on the name supplied when the array was created.

[list_begin definitions]

	[call [cmd name!]]
		Causes the array to come into existence in the current context.
		This is [cmd upvar] [arg #0] [arg varname] [arg name].
		From then on in the current context the array can be referenced by name.
		Returns an empty string.
		Nickname: Make Exist.

	[call [cmd name] [arg key] [opt [arg val]] [opt [arg {key val...}]]]
		Gets the value of one element of the array or sets the value of one or more elements of the array.
		The value(s) for the key(s) will be set using [cmd {array set}].
		Returns the value at [arg key].
		Nickname: Get/Set.

	[call [cmd name?] [arg key]]
		Returns true if the [arg key] exists in the array or false if the [arg key] doesn't exist in the array.
		Nickname: Exists.

	[call [cmd name-] [arg key] [opt [arg {key ...}]]]
		Removes key(s) from the array. The [cmd {array unset}] command is used, allowing key(s) to contain wildcards.
		It isn't an error if a key doesn't exist in the array.
		Returns an empty string.
		Nickname: Unset.

	[call [cmd name^] [arg key] [opt [arg default]] [opt [arg myVar]]]
		Returns the value at [arg key] if it exists,
		otherwise returns [arg default], or an empty string if [arg default] isn't supplied.
		If [arg myVar] is supplied and not an empty string then the variable it names
		will be set in the caller's context to the returned value.
		Nickname: Get With Default.

	[call [cmd name&] [opt [arg key]]]
		Returns the fully namespace qualified name of the array if no [arg key] supplied or if [arg key] is an empty string,
		otherwise returns the fully namespace qualified name of the array and the element [arg key].
		Nickname: Variable Reference.

	[call [cmd name&&]]
		Returns the fully namespace qualified command.
		Nickname: Command Reference.

	[call [cmd name=] [arg key] [opt [arg {key ...}]] [arg val]]
		Every [arg key] is set to [arg val].
		Returns [arg val].
		Nickname: Setto.
		
	[call [cmd name+] [arg key] [opt [arg {val ...}]]]
		Every [arg val] is lappended to the element at [arg key].
		Returns the value at [arg key].
		Nickname: Lappend.

	[call [cmd name+=] [arg key] [opt [arg incr]]]
		Increments the value at [arg key] by [arg incr] (default 1).
		Returns the value at [arg key].
		Nickname: Incr.

	[call [cmd name.] [arg key[opt op]] [opt [arg {arg ...}]]]
		The [arg key] should be another AitCS. Permits "nesting". Calls are chained.
		Nickname: Struct Dereference.

	[call [cmd name---]]
		Unsets the array and deletes all the commands.
		Returns an empty string.
		Nickname: Destroy.

[list_end]


[section EXAMPLES]

[example {

	# Set/Get/Variable Reference
	# Save a widget and link a textvariable
	aitcs new rec
	aitcs new gui
	set fld preference
	rec $fld Tcl
	gui $fld [entry .e -textvariable [rec& $fld]]
	[gui $fld] insert end /Tk
	string length [rec preference]; # 6

	# Get/Set
	aitcs new things
	things a 3
	things b [expr {[things a] + 2}]
	things c [expr {[things b] * [things a]}]
	things c; # 15
		
	# Setto/Exists/Unset/Variable Reference
	aitcs new fruits
	fruits= apple orange pear apricot 0
	if {![fruits? peach]} {
		fruits= peach 0
	}
	array names [fruits&]; # {apple orange pear apricot peach}
	fruits- p* a*
	array names [fruits&]; # {orange}

	# Get With Default
	aitcs new things
	things are good
	things^ are ok; # good
	things^ be ok; # ok
	things^ willbe; # Empty string
	things^ is nice here; # nice, and var here is set to nice

	# Lappend
	aitcs new store
	store this one
	store+ this two three four five
	store this; # {one two three four five}

	# Incr
	aitcs new counters
	counters cycles 0
	counters+= cycles
	counters+= cycles 2
	counters+= cycles -1
	counters cycles; # 2

	# Make Exist/Exists
	aitcs new cow
	cow!
	set cow(sound) moo
	if {[cow? sound] && $cow(sound) eq "moo"} {
		puts "Standard English Cow"
	}

	# Command Reference/Variable Reference/Incr
	aitcs new callback
	callback count 0
	after 1000 [list [callback&&]+= count]
	vwait [callback& count]
	callback count; # 1

	# Struct Dereference/Random Name/Initial Values
	aitcs new animals {} {} \
		dog [aitcs new {} {} {} says woof likes cat] \
		cat [aitcs new {} {} {} says meow likes dog]
	animals home [list dog cat]
	set res ""; foreach animal [animals home] {
		append res $animal " says " [animals. $animal says]
		append res " likes " [animals. $animal likes]
		append res " who says " [animals. [animals. $animal likes] says]\n
        }
	set res
	# dog says woof likes cat who says meow
	# cat says meow likes dog who says woof

	# Struct Dereference
	aitcs new outer
	aitcs new inner
	aitcs new innest
	inner count 6
	innest things [list]
	innest+ things 1 2
	outer member inner
	inner item innest
	outer. member+= count 2; # 8
	outer. member. item+ things 3 4; # {1 2 3 4}
	outer. member. item^ name; # Empty string

	# Destroy
	aitcs new tmp
	tmp count 5
	tmp+= count -1
	tmp count; # 4
	set fqarr [tmp&]
	info exists $fqarr; # 1
	tmp---
	info exists $fqarr; # 0
	tmp count; # <error>

}]

[manpage_end]
